/*
 * Created on 13/lug/06
 *
 */
package portachiavi;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.security.InvalidKeyException;
import java.security.NoSuchAlgorithmException;
import java.util.Iterator;
import java.util.Set;
import java.util.zip.ZipEntry;
import java.util.zip.ZipFile;
import java.util.zip.ZipOutputStream;

import javax.crypto.IllegalBlockSizeException;
import javax.crypto.NoSuchPaddingException;
import javax.swing.ImageIcon;
import javax.swing.JFileChooser;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.filechooser.FileFilter;
import javax.swing.filechooser.FileNameExtensionFilter;

import portachiavi.gui.DoublePasswordDialog;
import portachiavi.gui.EntryDialog;
import portachiavi.gui.PasswordDialog;
import portachiavi.gui.ShowEntryDialog;

/**
 * Controllore del portachiavi
 * 
 * @author andrea
 * @version 1.5
 * 
 */
public class Controller implements IController {

	private final String KEY_FILENAME = ".portachiavi.key";
	private File keyFile, objFile;
	private JList listViewer;
	private boolean modified;
	private char[] password;
	private IPortachiavi portachiavi;
	private final String PORTACHIAVI_FILENAME = ".portachiavi.obj";
	private JFileChooser textFileChooser = null;
	private final String USERDIR = System.getProperty("user.dir");
	private final String USERHOME = System.getProperty("user.home");
	private JFileChooser zipFileChooser = null;

	/**
	 * Costruttore
	 * 
	 * @param portachiavi
	 * @param listViewer
	 */
	public Controller(IPortachiavi portachiavi, JList listViewer) {
		this.portachiavi = portachiavi;
		this.listViewer = listViewer;
		objFile = new File(USERHOME, PORTACHIAVI_FILENAME);
		keyFile = new File(USERHOME, KEY_FILENAME);

		textFileChooser = new JFileChooser(USERDIR);
		FileFilter txtFilter = new FileNameExtensionFilter("Text File (*.txt)",
				"txt", "TXT");
		textFileChooser.setFileFilter(txtFilter);
		textFileChooser.setMultiSelectionEnabled(false);

		zipFileChooser = new JFileChooser(USERDIR);
		FileFilter zipFilter = new FileNameExtensionFilter(
				"ZIP Archive (*.zip)", "zip", "ZIP");
		zipFileChooser.setFileFilter(zipFilter);
		zipFileChooser.setMultiSelectionEnabled(false);

		autoLoadPortachiavi();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see portachiavi.IController#addEntry()
	 */
	public boolean addEntry() {
		EntryDialog ed = new EntryDialog();
		ed.setVisible(true);
		String id = ed.getId();
		String username = ed.getUsername();
		String password = ed.getPassword();
		if (id.length() != 0) {
			if (portachiavi.addEntry(id, new LoginData(username, password))) {
				printEntryList();
				modified = true;
				return true;
			} else {
				JOptionPane.showMessageDialog(null,
						"[ADD] Entry gia' presente!", "ERRORE",
						JOptionPane.ERROR_MESSAGE, new ImageIcon(getClass()
								.getResource("/data/error.png")));
				return false;
			}
		}
		return false;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see portachiavi.IController#autoLoadPortachiavi()
	 */
	public void autoLoadPortachiavi() {
		if (objFile.exists() && keyFile.exists()) {
			do {
				setPassword();
			} while (!loadPortachiavi());
		} else {
			setFirstPassword();
			savePortachiavi();
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see portachiavi.IController#changePassword()
	 */
	public boolean changePassword() {
		DoublePasswordDialog pd = new DoublePasswordDialog(
				"Cambiare la password?");
		pd.setVisible(true);
		password = pd.getPassword();
		if (password == null) {
			return false;
		} else {
			modified = true;
			return true;
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see portachiavi.IController#exit()
	 */
	public void exit() {
		if (modified) {
			int r = JOptionPane.showConfirmDialog(null,
					"Vuoi salvare prima di uscire?", "DOMANDA",
					JOptionPane.YES_NO_CANCEL_OPTION,
					JOptionPane.WARNING_MESSAGE, new ImageIcon(getClass()
							.getResource("/data/warning.png")));
			if (r == JOptionPane.YES_OPTION) {
				if (savePortachiavi()) {
					System.exit(0);
				}
			} else if (r == JOptionPane.NO_OPTION) {
				System.exit(0);
			} else if (r == JOptionPane.CANCEL_OPTION) {
				return;
			}
		} else {
			System.exit(0);
		}

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see portachiavi.IController#exportToArchive()
	 */
	public boolean exportToArchive() {
		int r = zipFileChooser.showSaveDialog(null);
		if (r == JFileChooser.APPROVE_OPTION) {
			File zipFile = zipFileChooser.getSelectedFile();

			// controllo la presenza dell'estensione nel file
			if (!(zipFile.getName().endsWith(".zip") || zipFile.getName()
					.endsWith(".ZIP"))) {
				zipFile = new File(zipFile.getAbsolutePath().concat(".zip"));
			}

			// creo l'archivio con dentro i due file del portachiavi
			byte[] buffer = new byte[1024];
			FileInputStream in = null;
			int len = 0;
			ZipOutputStream out = null;
			try {
				out = new ZipOutputStream(new FileOutputStream(zipFile));
				in = new FileInputStream(objFile);
				out.putNextEntry(new ZipEntry(PORTACHIAVI_FILENAME));
				while ((len = in.read(buffer)) > 0) {
					out.write(buffer, 0, len);
				}
				out.closeEntry();
				in.close();

				in = new FileInputStream(keyFile);
				out.putNextEntry(new ZipEntry(KEY_FILENAME));
				while ((len = in.read(buffer)) > 0) {
					out.write(buffer, 0, len);
				}
				out.closeEntry();
				in.close();
				out.close();
				return true;
			} catch (IOException ioe) {
				ioe.printStackTrace();
				return false;
			}
		} else {
			return false;
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see portachiavi.IController#exportToTextFile()
	 */
	public boolean exportToTextFile() {
		int r = textFileChooser.showSaveDialog(null);
		if (r == JFileChooser.APPROVE_OPTION) {
			File textFile = textFileChooser.getSelectedFile();
			try {
				portachiavi.export2TextFile(textFile);
				return true;
			} catch (IOException e) {
				JOptionPane.showMessageDialog(null,
						"[EXPORT] Errore di scrittura sul file.", "ERRORE",
						JOptionPane.ERROR_MESSAGE, new ImageIcon(getClass()
								.getResource("/data/error.png")));
				e.printStackTrace();
				return false;
			}
		}
		return false;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see portachiavi.IController#importFromArchive()
	 */
	public boolean importFromArchive() {
		int r = zipFileChooser.showOpenDialog(null);
		if (r == JFileChooser.APPROVE_OPTION) {
			File archiveFile = zipFileChooser.getSelectedFile();

			ZipEntry keyZipEntry = null;
			ZipEntry portachiaviZipEntry = null;
			ZipFile archive = null;
			try {
				archive = new ZipFile(archiveFile);
				keyZipEntry = archive.getEntry(KEY_FILENAME);
				portachiaviZipEntry = archive.getEntry(PORTACHIAVI_FILENAME);
				if ((keyZipEntry != null) && (portachiaviZipEntry != null)) {

					// se i file sono già presenti, faccio una copia di backup
					File keyFile = new File(USERHOME + "/" + KEY_FILENAME);
					if (keyFile.exists()) {
						File backupKeyFile = new File(USERHOME + "/"
								+ KEY_FILENAME + ".old");
						backupKeyFile.deleteOnExit();
						keyFile.renameTo(backupKeyFile);
					}
					File portachiaviFile = new File(USERHOME + "/"
							+ PORTACHIAVI_FILENAME);
					if (portachiaviFile.exists()) {
						File backupPortachiaviFile = new File(USERHOME + "/"
								+ PORTACHIAVI_FILENAME + ".old");
						backupPortachiaviFile.deleteOnExit();
						portachiaviFile.renameTo(backupPortachiaviFile);
					}

					// importo i due file del portachiavi
					InputStream is = null;
					FileOutputStream fos = null;
					byte[] buffer = new byte[1024];
					int number = 0;
					is = archive.getInputStream(keyZipEntry);
					fos = new FileOutputStream(keyFile);
					while ((number = is.read(buffer, 0, 1024)) > -1) {
						fos.write(buffer, 0, number);
					}
					is.close();
					fos.close();
					is = archive.getInputStream(portachiaviZipEntry);
					fos = new FileOutputStream(portachiaviFile);
					while ((number = is.read(buffer, 0, 1024)) > -1) {
						fos.write(buffer, 0, number);
					}
					is.close();
					fos.close();
					archive.close();
					return true;
				} else {
					archive.close();
					return false;
				}
			} catch (IOException e) {
				e.printStackTrace();
				return false;
			}
		} else {
			return false;
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see portachiavi.gui.IController#loadPortachiavi()
	 */
	public boolean loadPortachiavi() {
		try {
			portachiavi.loadList(password, objFile, keyFile);
			printEntryList();
			return true;
		} catch (InvalidKeyException e) {
			e.printStackTrace();
		} catch (NoSuchAlgorithmException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		} catch (ClassNotFoundException e) {
			e.printStackTrace();
		} catch (KeyLoadingException e) {
			e.printStackTrace();
			JOptionPane.showMessageDialog(null,
					"La password inserita non e' corretta. Riprova.", "ERRORE",
					JOptionPane.ERROR_MESSAGE, new ImageIcon(getClass()
							.getResource("/data/error.png")));
		}
		return false;

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see portachiavi.gui.IController#modificaEntry()
	 */
	public boolean modificaEntry() {
		String id = (String) listViewer.getSelectedValue();
		if (id != null) {
			try {
				ILoginData auth = portachiavi.searchEntry(id);
				EntryDialog ed = new EntryDialog(id, auth.getUser(), auth
						.getPassword(), auth.getTimestamp());
				ed.setVisible(true);
				portachiavi.removeEntry(id);
				portachiavi.addEntry(ed.getId(), new LoginData(
						ed.getUsername(), ed.getPassword()));
				printEntryList();
				modified = true;
				return true;
			} catch (EntryNotFoundException e) {
				JOptionPane.showMessageDialog(null, "[MODIFY] Entry "
						+ e.getMessage() + " non trovata", "ERRORE",
						JOptionPane.ERROR_MESSAGE, new ImageIcon(getClass()
								.getResource("/data/error.png")));
				e.printStackTrace();
			}
			return false;
		}
		JOptionPane.showMessageDialog(null,
				"[MODIFY] Nessuna riga selezionata", "ERRORE",
				JOptionPane.ERROR_MESSAGE, new ImageIcon(getClass()
						.getResource("/data/error.png")));
		return false;
	}

	private void printEntryList() {
		Set<String> set = portachiavi.getEntryList();
		String[] list = new String[set.size()];
		int j = 0;
		Iterator<String> i = set.iterator();
		while (i.hasNext()) {
			list[j] = i.next();
			j++;
		}
		java.util.Arrays.sort(list, String.CASE_INSENSITIVE_ORDER);
		listViewer.setListData(list);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see portachiavi.gui.IController#removeEntry()
	 */
	public boolean removeEntry() {
		String id = (String) listViewer.getSelectedValue();
		if (id != null) {
			int res = JOptionPane.showConfirmDialog(null,
					"Eliminare definitivamente la riga \"" + id + "\"?",
					"DOMANDA", JOptionPane.YES_NO_OPTION,
					JOptionPane.QUESTION_MESSAGE, new ImageIcon(getClass()
							.getResource("/data/warning.png")));
			if (res == JOptionPane.YES_OPTION) {
				try {
					portachiavi.removeEntry(id);
					printEntryList();
					modified = true;
					return true;
				} catch (EntryNotFoundException e) {
					JOptionPane.showMessageDialog(null, "[REMOVE] Entry "
							+ e.getMessage() + " non trovata", "ERRORE",
							JOptionPane.ERROR_MESSAGE, new ImageIcon(getClass()
									.getResource("/data/error.png")));
					e.printStackTrace();
				}
			}
			return false;
		}
		JOptionPane.showMessageDialog(null,
				"[REMOVE] Nessuna riga selezionata", "ERRORE",
				JOptionPane.ERROR_MESSAGE, new ImageIcon(getClass()
						.getResource("/data/error.png")));
		return false;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see portachiavi.gui.IController#savePortachiavi()
	 */
	public boolean savePortachiavi() {
		try {
			portachiavi.saveList(password, objFile, keyFile);
			modified = false;
			return true;
		} catch (InvalidKeyException e) {
			e.printStackTrace();
		} catch (NoSuchAlgorithmException e) {
			e.printStackTrace();
		} catch (NoSuchPaddingException e) {
			e.printStackTrace();
		} catch (IllegalBlockSizeException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		} catch (KeySavingException e) {
			e.printStackTrace();
		}
		return false;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see portachiavi.gui.IController#searchEntry()
	 */
	public boolean searchEntry() {
		String id = (String) listViewer.getSelectedValue();
		if (id != null) {
			try {
				ILoginData auth = portachiavi.searchEntry(id);
				ShowEntryDialog sed = new ShowEntryDialog(id, auth.getUser(),
						auth.getPassword(), auth.getTimestamp());
				sed.setVisible(true);
				return true;
			} catch (EntryNotFoundException e) {
				JOptionPane.showMessageDialog(null, "[SEARCH] Entry "
						+ e.getMessage() + " non trovata", "ERRORE",
						JOptionPane.ERROR_MESSAGE, new ImageIcon(getClass()
								.getResource("/data/error.png")));
				e.printStackTrace();
			}
			return false;
		}
		JOptionPane.showMessageDialog(null,
				"[SEARCH] Nessuna riga selezionata", "ERRORE",
				JOptionPane.ERROR_MESSAGE, new ImageIcon(getClass()
						.getResource("/data/error.png")));
		return false;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see portachiavi.gui.IController#setFirstPassword()
	 */
	public void setFirstPassword() {
		DoublePasswordDialog pd = new DoublePasswordDialog(
				"Scegli la password del nuovo portachiavi");
		pd.setVisible(true);
		password = pd.getPassword();
		if (password == null) {
			System.exit(0);
		} else {
			modified = true;
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see portachiavi.gui.IController#setPassword()
	 */
	public void setPassword() {
		PasswordDialog pd = new PasswordDialog();
		pd.setVisible(true);
		password = pd.getPassword();
		if (password == null) {
			System.exit(0);
		}
	}

}
