/*
 * Created on 13/lug/06
 *
 */
package portachiavi.gui;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Image;
import java.awt.Insets;
import java.awt.Toolkit;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;

import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JPasswordField;
import javax.swing.SwingConstants;

/**
 * 
 * @author andrea
 * @version 1.3
 */
public class DoublePasswordDialog extends JDialog implements KeyListener {

	private static final long serialVersionUID = 1L;
	private JButton jButton = null;
	private JPanel jContentPane = null;
	private JLabel jLabelIcon = null;
	private JPasswordField jPasswordField1 = null;
	private JPasswordField jPasswordField2 = null;
	private JPanel mainPanel = null;
	private char[] password;
	private String windowTitle = null;

	public DoublePasswordDialog(String windowTitle) {
		super();
		this.windowTitle = windowTitle;
		initialize();
	}

	private boolean equal(char[] a, char[] b) {
		if (a.length == 0) {
			return false;
		}
		if (a.length != b.length) {
			return false;
		}
		for (int i = 0; i < a.length; i++) {
			if (a[i] != b[i]) {
				return false;
			}
		}
		return true;
	}

	private JButton getJButton() {
		if (jButton == null) {
			jButton = new JButton();
			jButton.setText("OK");
			jButton.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					setPassword();
				}
			});
		}
		return jButton;
	}

	private JPanel getJContentPane() {
		if (jContentPane == null) {
			jContentPane = new JPanel();
			jContentPane.setLayout(new BorderLayout());
			jContentPane.add(getJLabelIcon(), BorderLayout.WEST);
			jContentPane.add(getJPanel(), BorderLayout.CENTER);
		}
		return jContentPane;
	}

	private JLabel getJLabelIcon() {
		if (jLabelIcon == null) {
			jLabelIcon = new JLabel();
			jLabelIcon.setText("");
			jLabelIcon.setPreferredSize(new Dimension(50, 50));
			jLabelIcon.setHorizontalTextPosition(SwingConstants.CENTER);
			jLabelIcon.setHorizontalAlignment(SwingConstants.CENTER);
			jLabelIcon.setIcon(new ImageIcon(getClass().getResource(
					"/data/password.png")));
		}
		return jLabelIcon;
	}

	private JPanel getJPanel() {
		if (mainPanel == null) {
			JLabel label1 = new JLabel("Inserisci la nuova password: ");
			JLabel label2 = new JLabel("Ripeti la nuova password: ");
			mainPanel = new JPanel();
			mainPanel.setLayout(new GridBagLayout());
			mainPanel.add(getJPasswordField1(), new GridBagConstraints(1, 0, 1,
					1, 1.0, 0.0, GridBagConstraints.CENTER,
					GridBagConstraints.BOTH, new Insets(0, 0, 0, 0), 0, 0));
			mainPanel.add(getJButton(), new GridBagConstraints(1, 2, 1, 1, 0.0,
					0.0, GridBagConstraints.CENTER,
					GridBagConstraints.VERTICAL, new Insets(0, 0, 0, 0), 0, 0));
			mainPanel.add(label1, new GridBagConstraints(0, 0, 1, 1, 0.0, 0.0,
					GridBagConstraints.WEST, GridBagConstraints.NONE,
					new Insets(0, 0, 0, 0), 0, 0));
			mainPanel.add(getJPasswordField2(), new GridBagConstraints(1, 1, 1,
					1, 1.0, 0.0, GridBagConstraints.CENTER,
					GridBagConstraints.BOTH, new Insets(0, 0, 0, 0), 0, 0));
			mainPanel.add(label2, new GridBagConstraints(0, 1, 1, 1, 0.0, 0.0,
					GridBagConstraints.WEST, GridBagConstraints.NONE,
					new Insets(0, 0, 0, 0), 0, 0));
		}
		return mainPanel;
	}

	/**
	 * This method initializes jPasswordField1
	 * 
	 * @return javax.swing.JPasswordField
	 */
	private JPasswordField getJPasswordField1() {
		if (jPasswordField1 == null) {
			jPasswordField1 = new JPasswordField();
			jPasswordField1.setColumns(20);
			jPasswordField1.addKeyListener(this);
		}
		return jPasswordField1;
	}

	private JPasswordField getJPasswordField2() {
		if (jPasswordField2 == null) {
			jPasswordField2 = new JPasswordField();
			jPasswordField2.setColumns(20);
			jPasswordField2.addKeyListener(this);
		}
		return jPasswordField2;
	}

	/**
	 * 
	 * @return password char sequence
	 */
	public char[] getPassword() {
		return password;
	}

	private void initialize() {
		setTitle(windowTitle);
		setContentPane(getJContentPane());
		setModal(true);
		this.setSize(500, 100);
		setLocationRelativeTo(null);
		Image icon = Toolkit.getDefaultToolkit().getImage(
				getClass().getResource("/data/app.png"));
		setIconImage(icon);
	}

	public void keyPressed(KeyEvent e) {
		if (e.getKeyCode() == KeyEvent.VK_ENTER) {
			setPassword();
		}
	}

	public void keyReleased(KeyEvent e) {
	}

	public void keyTyped(KeyEvent e) {
	}

	private boolean setPassword() {
		if (equal(getJPasswordField1().getPassword(), getJPasswordField2()
				.getPassword())) {
			password = getJPasswordField1().getPassword();
			setVisible(false);
			getJPasswordField1().setText("");
			getJPasswordField2().setText("");
			return true;
		} else {
			password = null;
			return false;
		}
	}

}
