/*
 * Created on 13/lug/06
 *
 */
package portachiavi.gui;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Event;
import java.awt.Image;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;

import javax.swing.BorderFactory;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JList;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSeparator;
import javax.swing.JTextField;
import javax.swing.JToolBar;
import javax.swing.KeyStroke;
import javax.swing.SwingConstants;

import portachiavi.IController;

/**
 * The main frame of the Portachiavi GUI
 * 
 * @author andrea
 * @version 1.5
 * 
 */
public class PortachiaviFrame extends JFrame {

	private static final String applicationName = "Portachiavi 1.5 beta";
	private static final long serialVersionUID = 1L;
	private JMenuItem aboutMenuItem = null;
	private JButton addButton;
	private JMenuItem addMenuItem = null;
	private JMenu advancedMenu = null;
	private IController controller;
	private JMenu entryMenu = null;
	private JMenuItem exitMenuItem = null;
	private JMenuItem exportTextFileMenuItem = null;
	private JMenuItem exportZipMenuItem = null;
	private JMenuItem importZipMenuItem = null;
	private JMenu infoMenu = null;
	private JList listViewer = null;
	private JPanel mainPanel = null;
	private JMenuBar menubar = null;
	private JButton modifyButton;
	private JMenuItem modifyMenuItem = null;
	private JButton openButton;
	private JMenuItem openMenuItem = null;
	private JMenuItem passwordMenuItem = null;
	private JMenu portachiaviMenu = null;
	private JButton removeButton;
	private JMenuItem removeMenuItem = null;
	private JButton saveButton;
	private JMenuItem saveMenuItem = null;
	private JButton showButton;
	private JMenuItem showMenuItem = null;
	private JTextField statusBar = null;
	private JToolBar toolBar;

	/**
	 * This is the default constructor
	 */
	public PortachiaviFrame(IController controller, JList listViewer) {
		super();
		this.controller = controller;
		this.listViewer = listViewer;
		initialize();
	}

	private void addEntry() {
		if (controller.addEntry()) {
			getStatusBar().setText("Entry aggiunta correttamente.");
		} else {
			getStatusBar().setText("Entry non aggiunta.");
		}
	}

	private void export2TextFile() {
		int resp = JOptionPane.showConfirmDialog(this,
				"Il file di testo in cui viene esportato\n"
						+ "il portachiavi non e' cifrato!\n\n"
						+ "Vuoi continuare?", "ATTENZIONE!",
				JOptionPane.YES_NO_OPTION, JOptionPane.INFORMATION_MESSAGE,
				new ImageIcon(getClass().getResource("/data/warning.png")));
		if (resp == JOptionPane.YES_OPTION) {
			if (controller.exportToTextFile()) {
				getStatusBar().setText("Portachiavi esportato correttamente.");
			} else {
				getStatusBar().setText("Portachiavi non esportato.");
			}
		} else {
			getStatusBar().setText("Portachiavi non esportato.");
		}
	}

	private JMenuItem getAboutMenuItem() {
		if (aboutMenuItem == null) {
			aboutMenuItem = new JMenuItem();
			aboutMenuItem.setText("Informazioni");
			aboutMenuItem.setIcon(new ImageIcon(getClass().getResource(
					"/data/info.png")));
			aboutMenuItem
					.addActionListener(new java.awt.event.ActionListener() {
						public void actionPerformed(java.awt.event.ActionEvent e) {
							showAboutPanel();
						}
					});
		}
		return aboutMenuItem;
	}

	private JButton getAddButton() {
		if (addButton == null) {
			addButton = new JButton();
			addButton.setIcon(new ImageIcon(getClass().getResource(
					"/data/add.png")));
			addButton
					.setToolTipText("Aggiungi una nuova coppia username-password");
			addButton.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					addEntry();
				}
			});
		}
		return addButton;
	}

	private JMenuItem getAddMenuItem() {
		if (addMenuItem == null) {
			addMenuItem = new JMenuItem();
			addMenuItem.setText("Aggiungi una coppia username/password");
			addMenuItem.setIcon(new ImageIcon(getClass().getResource(
					"/data/add.png")));
			addMenuItem.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					addEntry();
				}
			});
		}
		return addMenuItem;
	}

	/**
	 * This method initializes advancedMenu
	 * 
	 * @return javax.swing.JMenu
	 */
	private JMenu getAdvancedMenu() {
		if (advancedMenu == null) {
			advancedMenu = new JMenu();
			advancedMenu.setText("Opzioni avanzate");
			advancedMenu.setIcon(new ImageIcon(getClass().getResource(
					"/data/advanced.png")));
			advancedMenu.add(getImportZipMenuItem());
			advancedMenu.add(getExportZipMenuItem());
			advancedMenu.add(getExportTextFileMenuItem());
			advancedMenu.add(getPasswordMenuItem());
		}
		return advancedMenu;
	}

	private JMenu getEntryMenu() {
		if (entryMenu == null) {
			entryMenu = new JMenu();
			entryMenu.setText("Entry");
			entryMenu.add(getAddMenuItem());
			entryMenu.add(getRemoveMenuItem());
			entryMenu.add(getShowMenuItem());
			entryMenu.add(getModifyMenuItem());
		}
		return entryMenu;
	}

	private JMenuItem getExitMenuItem() {
		if (exitMenuItem == null) {
			exitMenuItem = new JMenuItem();
			exitMenuItem.setText("Esci dal programma");
			exitMenuItem.setIcon(new ImageIcon(getClass().getResource(
					"/data/exit.png")));
			exitMenuItem.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					controller.exit();
				}
			});
		}
		return exitMenuItem;
	}

	private JMenuItem getExportTextFileMenuItem() {
		if (exportTextFileMenuItem == null) {
			exportTextFileMenuItem = new JMenuItem();
			exportTextFileMenuItem.setText("Esporta su un file di testo");
			exportTextFileMenuItem.setIcon(new ImageIcon(getClass()
					.getResource("/data/export.png")));
			exportTextFileMenuItem
					.addActionListener(new java.awt.event.ActionListener() {
						public void actionPerformed(java.awt.event.ActionEvent e) {
							export2TextFile();
						}
					});
		}
		return exportTextFileMenuItem;
	}

	private JMenuItem getExportZipMenuItem() {
		if (exportZipMenuItem == null) {
			exportZipMenuItem = new JMenuItem();
			exportZipMenuItem.setText("Esporta su archivio zippato");
			exportZipMenuItem.setIcon(new ImageIcon(getClass().getResource(
					"/data/zip.png")));
			exportZipMenuItem
					.addActionListener(new java.awt.event.ActionListener() {
						public void actionPerformed(java.awt.event.ActionEvent e) {
							if (controller.exportToArchive()) {
								getStatusBar().setText(
										"Esportazione completata.");
							} else {
								getStatusBar().setText(
										"Esportazione annullata.");
							}
						}
					});
		}
		return exportZipMenuItem;
	}

	private JMenuItem getImportZipMenuItem() {
		if (importZipMenuItem == null) {
			importZipMenuItem = new JMenuItem();
			importZipMenuItem.setText("Importa da un archivio zippato");
			importZipMenuItem.setIcon(new ImageIcon(getClass().getResource(
					"/data/unzip.png")));
			importZipMenuItem
					.addActionListener(new java.awt.event.ActionListener() {
						public void actionPerformed(java.awt.event.ActionEvent e) {
							importFromArchive();
						}
					});
		}
		return importZipMenuItem;
	}

	private JMenu getInfoMenu() {
		if (infoMenu == null) {
			infoMenu = new JMenu();
			infoMenu.setText("?");
			infoMenu.add(getAboutMenuItem());
		}
		return infoMenu;
	}

	private JList getListViewer() {
		return listViewer;
	}

	private JPanel getMainPanel() {
		if (mainPanel == null) {
			mainPanel = new JPanel();
			mainPanel.setLayout(new BorderLayout());
			mainPanel.add(getStatusBar(), BorderLayout.SOUTH);
			mainPanel.add(getToolBar(), BorderLayout.EAST);
			JScrollPane scroller = new JScrollPane(getListViewer());
			scroller.setBorder(BorderFactory
					.createTitledBorder("Contenuto del portachiavi"));
			mainPanel.add(scroller, BorderLayout.CENTER);
		}
		return mainPanel;
	}

	private JMenuBar getMenubar() {
		if (menubar == null) {
			menubar = new JMenuBar();
			menubar.add(getPortachiaviMenu());
			menubar.add(getEntryMenu());
			menubar.add(getInfoMenu());
		}
		return menubar;
	}

	private JButton getModifyButton() {
		if (modifyButton == null) {
			modifyButton = new JButton();
			modifyButton.setIcon(new ImageIcon(getClass().getResource(
					"/data/modify.png")));
			modifyButton.setToolTipText("Modifica la riga selezionata");
			modifyButton.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					modifyEntry();
				}
			});
		}
		return modifyButton;
	}

	private JMenuItem getModifyMenuItem() {
		if (modifyMenuItem == null) {
			modifyMenuItem = new JMenuItem();
			modifyMenuItem
					.setText("Modifica username e password della riga selezionata");
			modifyMenuItem.setMnemonic(KeyEvent.VK_UNDEFINED);
			modifyMenuItem.setIcon(new ImageIcon(getClass().getResource(
					"/data/modify.png")));
			modifyMenuItem
					.addActionListener(new java.awt.event.ActionListener() {
						public void actionPerformed(java.awt.event.ActionEvent e) {
							modifyEntry();
						}
					});
		}
		return modifyMenuItem;
	}

	private JButton getOpenButton() {
		if (openButton == null) {
			openButton = new JButton();
			openButton.setIcon(new ImageIcon(getClass().getResource(
					"/data/open.png")));
			openButton.setToolTipText("Apri il portachiavi");
			openButton.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					openList();
				}
			});
		}
		return openButton;
	}

	private JMenuItem getOpenMenuItem() {
		if (openMenuItem == null) {
			openMenuItem = new JMenuItem();
			openMenuItem.setText("Carica il portachiavi da disco");
			openMenuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_O,
					Event.CTRL_MASK, true));
			openMenuItem.setIcon(new ImageIcon(getClass().getResource(
					"/data/open.png")));
			openMenuItem.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					openList();
				}
			});
		}
		return openMenuItem;
	}

	private JMenuItem getPasswordMenuItem() {
		if (passwordMenuItem == null) {
			passwordMenuItem = new JMenuItem();
			passwordMenuItem.setText("Cambia la password del portachiavi");
			passwordMenuItem.setIcon(new ImageIcon(getClass().getResource(
					"/data/password.png")));
			passwordMenuItem
					.addActionListener(new java.awt.event.ActionListener() {
						public void actionPerformed(java.awt.event.ActionEvent e) {
							controller.changePassword();
						}
					});
		}
		return passwordMenuItem;
	}

	private JMenu getPortachiaviMenu() {
		if (portachiaviMenu == null) {
			portachiaviMenu = new JMenu();
			portachiaviMenu.setText("Portachiavi");
			portachiaviMenu.add(getOpenMenuItem());
			portachiaviMenu.add(getSaveMenuItem());
			portachiaviMenu.add(new JSeparator());
			portachiaviMenu.add(getAdvancedMenu());
			portachiaviMenu.add(new JSeparator());
			portachiaviMenu.add(getExitMenuItem());
		}
		return portachiaviMenu;
	}

	private JButton getRemoveButton() {
		if (removeButton == null) {
			removeButton = new JButton();
			removeButton.setIcon(new ImageIcon(getClass().getResource(
					"/data/remove.png")));
			removeButton.setToolTipText("Cancella la riga selezionata");
			removeButton.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					removeEntry();
				}
			});
		}
		return removeButton;
	}

	private JMenuItem getRemoveMenuItem() {
		if (removeMenuItem == null) {
			removeMenuItem = new JMenuItem();
			removeMenuItem.setText("Elimina la riga selezionata");
			removeMenuItem.setAccelerator(KeyStroke.getKeyStroke(
					KeyEvent.VK_DELETE, Event.CTRL_MASK, true));
			removeMenuItem.setIcon(new ImageIcon(getClass().getResource(
					"/data/remove.png")));
			removeMenuItem
					.addActionListener(new java.awt.event.ActionListener() {
						public void actionPerformed(java.awt.event.ActionEvent e) {
							removeEntry();
						}
					});
		}
		return removeMenuItem;
	}

	private JButton getSaveButton() {
		if (saveButton == null) {
			saveButton = new JButton();
			saveButton.setIcon(new ImageIcon(getClass().getResource(
					"/data/save.png")));
			saveButton.setToolTipText("Salva il portachiavi");
			saveButton.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					saveList();
				}
			});
		}
		return saveButton;
	}

	private JMenuItem getSaveMenuItem() {
		if (saveMenuItem == null) {
			saveMenuItem = new JMenuItem();
			saveMenuItem.setText("Salva il portachiavi su disco");
			saveMenuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_S,
					Event.CTRL_MASK, true));
			saveMenuItem.setIcon(new ImageIcon(getClass().getResource(
					"/data/save.png")));
			saveMenuItem.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					saveList();
				}
			});
		}
		return saveMenuItem;
	}

	private JButton getShowButton() {
		if (showButton == null) {
			showButton = new JButton();
			showButton.setIcon(new ImageIcon(getClass().getResource(
					"/data/zoomin.png")));
			showButton
					.setToolTipText("Mostra username e password della riga selezionata");
			showButton.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					showEntry();
				}
			});
		}
		return showButton;
	}

	private JMenuItem getShowMenuItem() {
		if (showMenuItem == null) {
			showMenuItem = new JMenuItem();
			showMenuItem
					.setText("Visualizza username e password della riga selezionata");
			showMenuItem.setAccelerator(KeyStroke.getKeyStroke(
					KeyEvent.VK_ENTER, Event.CTRL_MASK, true));
			showMenuItem.setIcon(new ImageIcon(getClass().getResource(
					"/data/zoomin.png")));
			showMenuItem.setActionCommand("");
			showMenuItem.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					showEntry();
				}
			});
		}
		return showMenuItem;
	}

	private JTextField getStatusBar() {
		if (statusBar == null) {
			statusBar = new JTextField();
			statusBar.setEditable(false);
			statusBar.setForeground(Color.black);
			statusBar.setEnabled(true);
		}
		return statusBar;
	}

	private JToolBar getToolBar() {
		if (toolBar == null) {
			toolBar = new JToolBar();
			toolBar.setFloatable(false);
			toolBar.setOrientation(SwingConstants.VERTICAL);
			toolBar.add(getOpenButton());
			toolBar.add(getSaveButton());
			toolBar.addSeparator();
			toolBar.add(getAddButton());
			toolBar.add(getRemoveButton());
			toolBar.add(getShowButton());
			toolBar.add(getModifyButton());
		}
		return toolBar;
	}

	private void importFromArchive() {
		int resp = JOptionPane
				.showConfirmDialog(
						this,
						"1. Questa operazione è irreversibile.\n"
								+ "2. Questa operazione sovrascrive il portachiavi esistente.\n\n"
								+ "Vuoi continuare?", "ATTENZIONE!",
						JOptionPane.YES_NO_OPTION,
						JOptionPane.INFORMATION_MESSAGE, new ImageIcon(
								getClass().getResource("/data/warning.png")));
		if (resp == JOptionPane.YES_OPTION) {
			if (controller.importFromArchive() && controller.loadPortachiavi()) {
				getStatusBar().setText("Importazione eseguita.");
			} else {
				getStatusBar().setText("Importazione annullata.");
			}
		}
	}

	private void initialize() {
		setJMenuBar(getMenubar());
		setContentPane(getMainPanel());
		setTitle(applicationName);
		this.setSize(640, 480);
		setLocationRelativeTo(null);
		Image icon = Toolkit.getDefaultToolkit().getImage(
				getClass().getResource("/data/app.png"));
		setIconImage(icon);
		addWindowListener(new java.awt.event.WindowAdapter() {
			@Override
			public void windowClosing(java.awt.event.WindowEvent e) {
				controller.exit();
			}
		});
		listViewer.addMouseListener(new MouseAdapter() {
			@Override
			public void mouseClicked(MouseEvent e) {
				if (e.getButton() == MouseEvent.BUTTON1
						&& e.getClickCount() == 2) {
					showEntry();
				}
			}

		});
	}

	private void modifyEntry() {
		if (controller.modificaEntry()) {
			getStatusBar().setText("Modifica avvenuta correttamente.");
		} else {
			getStatusBar().setText("Modifica non avvenuta.");
		}
	}

	private void openList() {
		if (controller.loadPortachiavi()) {
			getStatusBar().setText("Portachiavi caricato correttamente.");
		} else {
			getStatusBar().setText("Portachiavi non caricato.");
		}
	}

	private void removeEntry() {
		if (controller.removeEntry()) {
			getStatusBar().setText("Rimozione della entry completata.");
		} else {
			getStatusBar().setText("Entry non rimossa.");
		}
	}

	private void saveList() {
		if (controller.savePortachiavi()) {
			getStatusBar().setText("Portachiavi salvato correttamente.");
		} else {
			getStatusBar().setText("Portachiavi non salvato.");
		}
	}

	private void showAboutPanel() {
		JOptionPane.showMessageDialog(null, applicationName
				+ "\nAndrea Urbini 2010\nhttp://www.urbo.altervista.org",
				"Informazioni", JOptionPane.INFORMATION_MESSAGE, new ImageIcon(
						getClass().getResource("/data/info.png")));
	}

	private void showEntry() {
		if (controller.searchEntry()) {
			getStatusBar().setText("Entry trovata.");
		} else {
			getStatusBar().setText("Entry non trovata.");
		}
	}
}